@@ -35,25 +35,24 @@ Q_PROPERTY(int startElements MEMBER m_startElements)
 public:
     explicit DummyBackend(QObject* parent = nullptr);
 
-    void setMetaData(const QString& path) override;
-    QList<AbstractResource*> upgradeablePackages() const override;
-    AbstractResource* resourceByPackageName(const QString& name) const override;
     int updatesCount() const override;
     AbstractBackendUpdater* backendUpdater() const override;
     AbstractReviewsBackend* reviewsBackend() const override;
-    QList<AbstractResource*> searchPackageName(const QString& searchText) override;
-    QVector<AbstractResource*> allResources() const override;
+    ResultsStream* search(const AbstractResourcesBackend::Filters & search) override;
+    ResultsStream * findResourceByPackageName(const QUrl& search) override;
+    QHash<QString, DummyResource*> resources() const { return m_resources; }
     bool isValid() const override { return true; } // No external file dependencies that could cause runtime errors
-    QList<QAction*> messageActions() const override { return m_messageActions; }
 
-    void cancelTransaction(AbstractResource* app) override;
-    void installApplication(AbstractResource* app) override;
-    void installApplication(AbstractResource* app, const AddonList& addons) override;
-    void removeApplication(AbstractResource* app) override;
+    Transaction* installApplication(AbstractResource* app) override;
+    Transaction* installApplication(AbstractResource* app, const AddonList& addons) override;
+    Transaction* removeApplication(AbstractResource* app) override;
     bool isFetching() const override { return m_fetching; }
+    AbstractResource * resourceForFile(const QUrl & ) override;
+    void checkForUpdates() override;
+    QString displayName() const override;
+    bool hasApplications() const override;
 
 public Q_SLOTS:
-    void checkForUpdates();
     void toggleFetching();
 
 private:
@@ -64,7 +63,6 @@ private:
     DummyReviewsBackend* m_reviews;
     bool m_fetching;
     int m_startElements;
-    QList<QAction*> m_messageActions;
 };
 
 #endif // DUMMYBACKEND_H
