@@ -32,8 +32,6 @@
 #include <QDebug>
 #include <QDesktopServices>
 
-Q_DECLARE_METATYPE(AbstractResource*)
-
 class SharedManager : public QObject
 {
 Q_OBJECT
@@ -69,8 +67,7 @@ Rating* KNSReviews::ratingForApplication(AbstractResource* app) const
     return new Rating(
         resource->packageName(),
         noc,
-        rating/10,
-        QLatin1Char('[')+QString::number(noc*rating)+QLatin1Char(']')
+        { { QStringLiteral("star5"), rating } }
     );
 }
 
@@ -85,18 +82,22 @@ void KNSReviews::fetchReviews(AbstractResource* app, int page)
     job->setProperty("app", qVariantFromValue<AbstractResource*>(app));
     connect(job, &Attica::BaseJob::finished, this, &KNSReviews::commentsReceived);
     job->start();
+    m_fetching++;
 }
 
 void KNSReviews::commentsReceived(Attica::BaseJob* j)
 {
+    m_fetching--;
     Attica::ListJob<Attica::Comment>* job = static_cast<Attica::ListJob<Attica::Comment>*>(j);
     Attica::Comment::List comments = job->itemList();
 
     QVector<ReviewPtr> reviews;
+    reviews.reserve(comments.count());
     AbstractResource* app = job->property("app").value<AbstractResource*>();
     foreach(const Attica::Comment& comment, comments) {
         //TODO: language lookup?
-        ReviewPtr r(new Review(app->name(), app->packageName(), QStringLiteral("en"), comment.subject(), comment.text(), comment.user(),
+        const QString subject = comment.subject().isEmpty() ? QStringLiteral(".") : comment.subject();
+        ReviewPtr r(new Review(app->name(), app->packageName(), QStringLiteral("en"), subject, comment.text(), comment.user(),
             comment.date(), true, comment.id().toInt(), comment.score()/10, 0, 0, QString()
         ));
         reviews += r;
@@ -107,7 +108,7 @@ void KNSReviews::commentsReceived(Attica::BaseJob* j)
 
 bool KNSReviews::isFetching() const
 {
-    return m_backend->isFetching();
+    return m_fetching > 0;
 }
 
 void KNSReviews::flagReview(Review*  /*r*/, const QString&  /*reason*/, const QString&  /*text*/)
