@@ -22,20 +22,21 @@
 #include "PackageKitResource.h"
 #include "PackageKitBackend.h"
 #include "PackageKitMessages.h"
-#include <MuonDataSources.h>
 #include <KLocalizedString>
 #include <PackageKit/Details>
 #include <PackageKit/Daemon>
+#include <QDebug>
 
 PackageKitResource::PackageKitResource(QString packageName, QString summary, PackageKitBackend* parent)
     : AbstractResource(parent)
     , m_summary(std::move(summary))
     , m_name(std::move(packageName))
+    , m_dependenciesCount(0)
 {
     setObjectName(m_name);
 }
 
-QString PackageKitResource::name()
+QString PackageKitResource::name() const
 {
     return m_name;
 }
@@ -59,14 +60,14 @@ QString PackageKitResource::availablePackageId() const
 
     QMap<PackageKit::Transaction::Info, QStringList>::const_iterator it = m_packages.constFind(PackageKit::Transaction::InfoAvailable);
     if (it != m_packages.constEnd())
-        return it->first();
+        return it->last();
     return installedPackageId();
 }
 
 QString PackageKitResource::installedPackageId() const
 {
     const auto installed = m_packages[PackageKit::Transaction::InfoInstalled];
-    return installed.isEmpty() ? QString() : installed.first();
+    return installed.isEmpty() ? QString() : installed.last();
 }
 
 QString PackageKitResource::comment()
@@ -86,7 +87,7 @@ QUrl PackageKitResource::homepage()
     return QUrl(m_details.url());
 }
 
-QString PackageKitResource::icon() const
+QVariant PackageKitResource::icon() const
 {
     return QStringLiteral("applications-other");
 }
@@ -120,8 +121,8 @@ int PackageKitResource::size()
 
 QString PackageKitResource::origin() const
 {
-    //TODO
-    return QStringLiteral("PackageKit");
+    auto pkgid = availablePackageId();
+    return PackageKit::Daemon::packageData(pkgid);
 }
 
 QString PackageKitResource::section()
@@ -129,17 +130,7 @@ QString PackageKitResource::section()
     return QString();
 }
 
-QUrl PackageKitResource::screenshotUrl()
-{
-    return {};
-}
-
-QUrl PackageKitResource::thumbnailUrl()
-{
-    return {};
-}
-
-AbstractResource::State PackageKitResource::state()
+AbstractResource::State PackageKitResource::state() const
 {
     if (backend()->isPackageNameUpgradeable(this))
         return Upgradeable;
@@ -151,17 +142,12 @@ AbstractResource::State PackageKitResource::state()
         return Broken;
 }
 
-void PackageKitResource::setPackages(const QMap<PackageKit::Transaction::Info, QStringList> &packages)
+void PackageKitResource::addPackageId(PackageKit::Transaction::Info info, const QString &packageId, bool arch)
 {
-    if (m_packages != packages) {
-        m_packages = packages;
-        emit stateChanged();
-    }
-}
-
-void PackageKitResource::addPackageId(PackageKit::Transaction::Info info, const QString &packageId)
-{
-    m_packages[info].append(packageId);
+    if (arch)
+        m_packages[info].append(packageId);
+    else
+        m_packages[info].prepend(packageId);
     emit stateChanged();
 }
 
@@ -172,18 +158,17 @@ QStringList PackageKitResource::categories()
 
 bool PackageKitResource::isTechnical() const
 {
-    return true;//!m_availablePackageId.startsWith("flash");
+    return true;
 }
 
 void PackageKitResource::fetchDetails()
 {
-    if (!m_details.isEmpty())
+    const QString pkgid = availablePackageId();
+    if (!m_details.isEmpty() || pkgid.isEmpty())
         return;
     m_details.insert(QStringLiteral("fetching"), true);//we add an entry so it's not re-fetched.
 
-    PackageKit::Transaction* t = PackageKit::Daemon::getDetails(availablePackageId());
-    connect(t, &PackageKit::Transaction::details, this, &PackageKitResource::setDetails);
-    connect(t, &PackageKit::Transaction::errorCode, this, &PackageKitResource::failedFetchingDetails);
+    backend()->fetchDetails(pkgid);
 }
 
 void PackageKitResource::failedFetchingDetails(PackageKit::Transaction::Error, const QString& msg)
@@ -191,6 +176,14 @@ void PackageKitResource::failedFetchingDetails(PackageKit::Transaction::Error, c
     qWarning() << "error fetching details" << msg;
 }
 
+void PackageKitResource::setDependenciesCount(uint deps)
+{
+    if (deps != m_dependenciesCount) {
+        m_dependenciesCount = deps;
+        Q_EMIT sizeChanged();
+    }
+}
+
 void PackageKitResource::setDetails(const PackageKit::Details & details)
 {
     const bool ourDetails = details.packageId() == availablePackageId();
@@ -216,16 +209,17 @@ void PackageKitResource::fetchChangelog()
 static void addIfNotEmpty(const QString& title, const QString& content, QString& where)
 {
     if (!content.isEmpty())
-        where += QStringLiteral("<p><b>") + title + QStringLiteral("</b>&nbsp;") + content + QStringLiteral("</p>");
+        where += QStringLiteral("<p><b>") + title + QStringLiteral("</b>&nbsp;") + QString(content).replace(QStringLiteral("\n"), QStringLiteral("<br />")) + QStringLiteral("</p>");
 }
 
-static QString joinPackages(const QStringList& pkgids)
+QString PackageKitResource::joinPackages(const QStringList& pkgids, const QString &_sep)
 {
     QStringList ret;
     foreach(const QString& pkgid, pkgids) {
         ret += i18nc("package-name (version)", "%1 (%2)", PackageKit::Daemon::packageName(pkgid), PackageKit::Daemon::packageVersion(pkgid));
     }
-    return ret.join(i18nc("comma separating package names", ", "));
+    const QString sep = _sep.isEmpty() ? i18nc("comma separating package names", ", ") : _sep;
+    return ret.join(sep);
 }
 
 static QStringList urlToLinks(const QStringList& urls)
@@ -244,7 +238,6 @@ void PackageKitResource::updateDetail(const QString& /*packageID*/, const QStrin
     addIfNotEmpty(i18n("Reason:"), updateText, info);
     addIfNotEmpty(i18n("Obsoletes:"), joinPackages(obsoletes), info);
     addIfNotEmpty(i18n("Updates:"), joinPackages(updates), info);
-    addIfNotEmpty(i18n("Change Log:"), changelog, info);
     addIfNotEmpty(i18n("Update State:"), PackageKitMessages::updateStateMessage(state), info);
     addIfNotEmpty(i18n("Restart:"), PackageKitMessages::restartMessage(restart), info);
 
@@ -259,3 +252,10 @@ PackageKitBackend* PackageKitResource::backend() const
     return qobject_cast<PackageKitBackend*>(parent());
 }
 
+QString PackageKitResource::sizeDescription()
+{
+    if (m_dependenciesCount == 0)
+        return AbstractResource::sizeDescription();
+    else
+        return i18np("%2 (plus %1 dependency)", "%2 (plus %1 dependencies)", m_dependenciesCount, AbstractResource::sizeDescription());
+}
