@@ -33,7 +33,7 @@ class PackageKitResource : public AbstractResource
     public:
         explicit PackageKitResource(QString  packageName, QString  summary, PackageKitBackend* parent);
         QString packageName() const override;
-        QString name() override;
+        QString name() const override;
         QString comment() override;
         QString longDescription() override;
         QUrl homepage() override;
@@ -47,10 +47,7 @@ class PackageKitResource : public AbstractResource
         void fetchChangelog() override;
         
         QList<PackageState> addonsInformation() override;
-        State state() override;
-        
-        QUrl screenshotUrl() override;
-        QUrl thumbnailUrl() override;
+        State state() const override;
         
         QString installedVersion() const override;
         QString availableVersion() const override;
@@ -64,7 +61,13 @@ class PackageKitResource : public AbstractResource
 
         PackageKitBackend* backend() const;
 
-        static QString joinPackages(const QStringList& pkgids);
+        static QString joinPackages(const QStringList& pkgids, const QString &_sep = {});
+
+        void invokeApplication() const override {}
+        bool canExecute() const override { return false; }
+
+        QString sizeDescription() override;
+        void setDependenciesCount(uint count);
 
     public Q_SLOTS:
         void addPackageId(PackageKit::Transaction::Info info, const QString &packageId, bool arch);
@@ -94,6 +97,7 @@ class PackageKitResource : public AbstractResource
         const QString m_summary;
         const QString m_name;
         PackageKit::Details m_details;
+        uint m_dependenciesCount;
 };
 
 #endif // PACKAGEKITRESOURCE_H
