@@ -21,12 +21,33 @@
 #include "PackageKitMessages.h"
 
 #include <PackageKit/Daemon>
+#ifdef PKQT_1_0
+#include <PackageKit/Offline>
+#endif
 #include <QDebug>
 #include <QAction>
 #include <QSet>
 
 #include <KLocalizedString>
-#include <QIcon>
+
+static int percentageWithStatus(PackageKit::Transaction::Status status, uint percentage)
+{
+    if (status != PackageKit::Transaction::StatusUnknown) {
+        static const QMap<PackageKit::Transaction::Status, int> statuses = {
+            { PackageKit::Transaction::Status::StatusDownload, 0 },
+            { PackageKit::Transaction::Status::StatusInstall, 1},
+            { PackageKit::Transaction::Status::StatusUpdate, 1}
+        };
+        const auto idx = statuses.value(status, -1);
+        if (idx < 0) {
+            qDebug() << "Status not present" << status << "among" << statuses   .keys() << percentage;
+            return -1;
+        }
+        percentage = (idx * 100 + percentage) / 2 /*the maximum in statuses*/;
+    }
+    qDebug() << "reporing progress with status:" << status << percentage;
+    return percentage;
+}
 
 PackageKitUpdater::PackageKitUpdater(PackageKitBackend * parent)
   : AbstractBackendUpdater(parent),
@@ -34,8 +55,6 @@ PackageKitUpdater::PackageKitUpdater(PackageKitBackend * parent)
     m_backend(parent),
     m_isCancelable(false),
     m_isProgressing(false),
-    m_speed(0),
-    m_remainingTime(0),
     m_percentage(0),
     m_lastUpdate()
 {
@@ -44,7 +63,6 @@ PackageKitUpdater::PackageKitUpdater(PackageKitBackend * parent)
 
 PackageKitUpdater::~PackageKitUpdater()
 {
-    delete m_transaction;
 }
 
 void PackageKitUpdater::prepare()
@@ -57,20 +75,19 @@ void PackageKitUpdater::prepare()
 void PackageKitUpdater::setupTransaction(PackageKit::Transaction::TransactionFlags flags)
 {
     m_packagesRemoved.clear();
-    m_transaction = PackageKit::Daemon::updatePackages(involvedPackages(m_toUpgrade).toList(), flags);
+    auto pkgs = involvedPackages(m_toUpgrade).toList();
+    pkgs.sort();
+    m_transaction = PackageKit::Daemon::updatePackages(pkgs, flags);
     m_isCancelable = m_transaction->allowCancel();
 
     connect(m_transaction.data(), &PackageKit::Transaction::finished, this, &PackageKitUpdater::finished);
     connect(m_transaction.data(), &PackageKit::Transaction::package, this, &PackageKitUpdater::packageResolved);
     connect(m_transaction.data(), &PackageKit::Transaction::errorCode, this, &PackageKitUpdater::errorFound);
-    connect(m_transaction.data(), &PackageKit::Transaction::message, this, &PackageKitUpdater::printMessage);
     connect(m_transaction.data(), &PackageKit::Transaction::mediaChangeRequired, this, &PackageKitUpdater::mediaChange);
     connect(m_transaction.data(), &PackageKit::Transaction::requireRestart, this, &PackageKitUpdater::requireRestart);
     connect(m_transaction.data(), &PackageKit::Transaction::eulaRequired, this, &PackageKitUpdater::eulaRequired);
-    connect(m_transaction.data(), &PackageKit::Transaction::statusChanged, this, &PackageKitUpdater::statusChanged);
-    connect(m_transaction.data(), &PackageKit::Transaction::speedChanged, this, &PackageKitUpdater::speedChanged);
+    connect(m_transaction.data(), &PackageKit::Transaction::repoSignatureRequired, this, &PackageKitUpdater::repoSignatureRequired);
     connect(m_transaction.data(), &PackageKit::Transaction::allowCancelChanged, this, &PackageKitUpdater::cancellableChanged);
-    connect(m_transaction.data(), &PackageKit::Transaction::remainingTimeChanged, this, &PackageKitUpdater::remainingTimeChanged);
     connect(m_transaction.data(), &PackageKit::Transaction::percentageChanged, this, &PackageKitUpdater::percentageChanged);
     connect(m_transaction.data(), &PackageKit::Transaction::itemProgress, this, &PackageKitUpdater::itemProgress);
 }
@@ -106,14 +123,34 @@ QSet<QString> PackageKitUpdater::involvedPackages(const QSet<AbstractResource*>&
     return packageIds;
 }
 
+void PackageKitUpdater::processProceedFunction()
+{
+    auto t = m_proceedFunctions.takeFirst()();
+    connect(t, &PackageKit::Transaction::finished, this, [this](PackageKit::Transaction::Exit status) {
+        if (status != PackageKit::Transaction::Exit::ExitSuccess) {
+            qWarning() << "transaction failed" << sender() << status;
+            cancel();
+            return;
+        }
+
+        if (!m_proceedFunctions.isEmpty()) {
+            processProceedFunction();
+        } else {
+            start();
+        }
+    });
+}
+
 void PackageKitUpdater::proceed()
 {
-    if (!m_requiredEula.isEmpty()) {
-        PackageKit::Transaction* t = PackageKit::Daemon::acceptEula(m_requiredEula.takeFirst());
-        connect(t, &PackageKit::Transaction::finished, this, &PackageKitUpdater::start);
-        return;
-    }
-    setupTransaction(PackageKit::Transaction::TransactionFlagOnlyTrusted);
+    if (!m_proceedFunctions.isEmpty())
+        processProceedFunction();
+#ifdef PKQT_1_0
+    else if (qEnvironmentVariableIsSet("PK_OFFLINE_UPDATE"))
+        setupTransaction(PackageKit::Transaction::TransactionFlagOnlyTrusted | PackageKit::Transaction::TransactionFlagOnlyDownload);
+#endif
+    else
+        setupTransaction(PackageKit::Transaction::TransactionFlagOnlyTrusted);
 }
 
 void PackageKitUpdater::start()
@@ -127,7 +164,7 @@ void PackageKitUpdater::start()
 void PackageKitUpdater::finished(PackageKit::Transaction::Exit exit, uint /*time*/)
 {
 //     qDebug() << "update finished!" << exit << time;
-    if (exit == PackageKit::Transaction::ExitEulaRequired)
+    if (!m_proceedFunctions.isEmpty())
         return;
     const bool cancel = exit == PackageKit::Transaction::ExitCancelled;
     const bool simulate = m_transaction->transactionFlags() & PackageKit::Transaction::TransactionFlagSimulate;
@@ -137,9 +174,8 @@ void PackageKitUpdater::finished(PackageKit::Transaction::Exit exit, uint /*time
 
     if (!cancel && simulate) {
         if (!m_packagesRemoved.isEmpty())
-            Q_EMIT proceedRequest(i18n("Packages to remove"), i18n("The following packages will be removed by the update:\n%1", PackageKitResource::joinPackages(m_packagesRemoved)));
+            Q_EMIT proceedRequest(i18n("Packages to remove"), i18n("The following packages will be removed by the update:\n<ul><li>%1</li></ul>", PackageKitResource::joinPackages(m_packagesRemoved, QStringLiteral("</li><li>"))));
         else {
-            Q_ASSERT(m_requiredEula.isEmpty());
             proceed();
         }
         return;
@@ -148,6 +184,13 @@ void PackageKitUpdater::finished(PackageKit::Transaction::Exit exit, uint /*time
     setProgressing(false);
     m_backend->refreshDatabase();
     fetchLastUpdateTime();
+
+    if (qEnvironmentVariableIsSet("PK_OFFLINE_UPDATE"))
+#ifdef PKQT_1_0
+        PackageKit::Daemon::global()->offline()->trigger(PackageKit::Offline::ActionReboot);
+#else
+        qWarning() << "PK_OFFLINE_UPDATE is set but discover was built against an old version of PackageKitQt that didn't support offline updates";
+#endif
 }
 
 void PackageKitUpdater::cancellableChanged()
@@ -160,40 +203,13 @@ void PackageKitUpdater::cancellableChanged()
 
 void PackageKitUpdater::percentageChanged()
 {
-    if (m_percentage != m_transaction->percentage()) {
-        m_percentage = m_transaction->percentage();
+    const auto actualPercentage = percentageWithStatus(m_transaction->status(), m_transaction->percentage());
+    if (actualPercentage >= 0 && m_percentage != actualPercentage) {
+        m_percentage = actualPercentage;
         emit progressChanged(m_percentage);
     }
 }
 
-void PackageKitUpdater::remainingTimeChanged()
-{
-    if (m_remainingTime != m_transaction->remainingTime()) {
-        m_remainingTime = m_transaction->remainingTime();
-        emit remainingTimeChanged();
-    }
-}
-
-void PackageKitUpdater::speedChanged()
-{
-    if (m_speed != m_transaction->speed()) {
-        m_speed = m_transaction->speed();
-        emit downloadSpeedChanged(m_speed);
-    }
-}
-
-void PackageKitUpdater::statusChanged()
-{
-    if (m_status != m_transaction->status()) {
-        m_status = m_transaction->status();
-        m_statusMessage = PackageKitMessages::statusMessage(m_status);
-        m_statusDetail = PackageKitMessages::statusDetail(m_status);
-
-        emit statusMessageChanged(m_statusMessage);
-        emit statusDetailChanged(m_statusDetail);
-    }
-}
-
 bool PackageKitUpdater::hasUpdates() const
 {
     return m_backend->updatesCount() > 0;
@@ -204,12 +220,6 @@ qreal PackageKitUpdater::progress() const
     return m_percentage;
 }
 
-/** proposed ETA in milliseconds */
-long unsigned int PackageKitUpdater::remainingTime() const
-{
-    return m_remainingTime;
-}
-
 void PackageKitUpdater::removeResources(const QList<AbstractResource*>& apps)
 {
     QSet<QString> pkgs = involvedPackages(apps.toSet());
@@ -247,21 +257,6 @@ bool PackageKitUpdater::isProgressing() const
     return m_isProgressing;
 }
 
-QString PackageKitUpdater::statusMessage() const
-{
-    return m_statusMessage;
-}
-
-QString PackageKitUpdater::statusDetail() const
-{
-    return m_statusDetail;
-}
-
-quint64 PackageKitUpdater::downloadSpeed() const
-{
-    return m_speed;
-}
-
 void PackageKitUpdater::cancel()
 {
     if (m_transaction)
@@ -272,18 +267,12 @@ void PackageKitUpdater::cancel()
 
 void PackageKitUpdater::errorFound(PackageKit::Transaction::Error err, const QString& error)
 {
-    Q_UNUSED(error);
     if (err == PackageKit::Transaction::ErrorNoLicenseAgreement)
         return;
     Q_EMIT passiveMessage(QStringLiteral("%1\n%2").arg(PackageKitMessages::errorMessage(err), error));
     qWarning() << "Error happened" << err << error;
 }
 
-void PackageKitUpdater::printMessage(PackageKit::Transaction::Message type, const QString &message)
-{
-    qDebug() << "message" << type << message;
-}
-
 void PackageKitUpdater::mediaChange(PackageKit::Transaction::MediaType media, const QString& type, const QString& text)
 {
     Q_UNUSED(media)
@@ -297,7 +286,9 @@ void PackageKitUpdater::requireRestart(PackageKit::Transaction::Restart restart,
 
 void PackageKitUpdater::eulaRequired(const QString& eulaID, const QString& packageID, const QString& vendor, const QString& licenseAgreement)
 {
-    m_requiredEula += eulaID;
+    m_proceedFunctions << [eulaID](){
+        return PackageKit::Daemon::acceptEula(eulaID);
+    };
     Q_EMIT proceedRequest(i18n("Accept EULA"), i18n("The package %1 and its vendor %2 require that you accept their license:\n %3",
                                                  PackageKit::Daemon::packageName(packageID), vendor, licenseAgreement));
 }
@@ -328,12 +319,16 @@ void PackageKitUpdater::lastUpdateTimeReceived(QDBusPendingCallWatcher* w)
     w->deleteLater();
 }
 
-void PackageKitUpdater::itemProgress(const QString& itemID, PackageKit::Transaction::Status /*status*/, uint percentage)
+void PackageKitUpdater::itemProgress(const QString& itemID, PackageKit::Transaction::Status status, uint percentage)
 {
     auto res = packagesForPackageId({itemID});
 
+    const auto actualPercentage = percentageWithStatus(status, percentage);
+    if (actualPercentage<0)
+        return;
+
     foreach(auto r, res) {
-        Q_EMIT resourceProgressed(r, percentage);
+        Q_EMIT resourceProgressed(r, actualPercentage);
     }
 }
 
@@ -366,3 +361,31 @@ void PackageKitUpdater::packageResolved(PackageKit::Transaction::Info info, cons
     if (info == PackageKit::Transaction::InfoRemoving)
         m_packagesRemoved << packageId;
 }
+
+void PackageKitUpdater::repoSignatureRequired(const QString& packageID, const QString& repoName, const QString& keyUrl,
+                                              const QString& keyUserid, const QString& keyId, const QString& keyFingerprint,
+                                              const QString& keyTimestamp, PackageKit::Transaction::SigType type)
+{
+    Q_EMIT proceedRequest(i18n("Missing signature for %1 in %2", packageID, repoName),
+                          i18n("Do you trust the following key?\n\nUrl: %1\nUser: %2\nKey: %3\nFingerprint: %4\nTimestamp: %4\n",
+                               keyUrl, keyUserid, keyFingerprint, keyTimestamp));
+
+    m_proceedFunctions << [type, keyId, packageID](){
+        return PackageKit::Daemon::installSignature(type, keyId, packageID);
+    };
+}
+
+double PackageKitUpdater::updateSize() const
+{
+    double ret = 0.;
+    QSet<QString> donePkgs;
+    for (AbstractResource * res : m_toUpgrade) {
+        PackageKitResource * app = qobject_cast<PackageKitResource*>(res);
+        QString pkgid = m_backend->upgradeablePackageId(app);
+        if (!donePkgs.contains(pkgid)) {
+            donePkgs.insert(pkgid);
+            ret += app->size();
+        }
+    }
+    return ret;
+}
