@@ -26,7 +26,7 @@
 #include <resources/SourcesModel.h>
 #include <Category/Category.h>
 #include <Transaction/Transaction.h>
-#include <Transaction/TransactionModel.h>
+#include <resources/StoredResultsStream.h>
 
 #include <KAboutData>
 #include <KLocalizedString>
@@ -38,14 +38,30 @@
 #include <QTimer>
 #include <QAction>
 
-MUON_BACKEND_PLUGIN(SnapBackend)
+#include "utils.h"
+
+DISCOVER_BACKEND_PLUGIN(SnapBackend)
 
 SnapBackend::SnapBackend(QObject* parent)
     : AbstractResourcesBackend(parent)
     , m_updater(new StandardBackendUpdater(this))
     , m_reviews(new SnapReviewsBackend(this))
 {
+    {
+        auto request = m_client.connect();
+        request->runSync();
+        m_valid = request->error() == QSnapdRequest::NoError;
+        if (!m_valid) {
+            qWarning() << "snap problem at initialize:" << request->errorString();
+            return;
+        }
+    }
     connect(m_reviews, &SnapReviewsBackend::ratingsReady, this, &AbstractResourcesBackend::emitRatingsReady);
+
+    //make sure we populate the installed resources first
+    refreshStates();
+
+    SourcesModel::global()->addBackend(this);
 }
 
 int SnapBackend::updatesCount() const
@@ -57,56 +73,61 @@ static ResultsStream* voidStream() { return new ResultsStream(QStringLiteral("Sn
 
 ResultsStream * SnapBackend::search(const AbstractResourcesBackend::Filters& filters)
 {
-    if (filters.category && filters.category->isAddons())
+    if (!filters.extends.isEmpty()) {
         return voidStream();
-    if (filters.state >= AbstractResource::Installed) {
-        return populate(m_socket.snaps(), AbstractResource::Installed);
-    } else {
-        return populate(m_socket.find(filters.search), AbstractResource::None);
+    } else if (!filters.resourceUrl.isEmpty()) {
+        return findResourceByPackageName(filters.resourceUrl);
+    } else if (filters.category && filters.category->isAddons()) {
+        return voidStream();
+    } else if (filters.state >= AbstractResource::Installed) {
+        return populate(m_client.list());
+    } else if (!filters.search.isEmpty()) {
+        return populate(m_client.find(QSnapdClient::FindFlag::None, filters.search));
     }
     return voidStream();
 }
 
 ResultsStream * SnapBackend::findResourceByPackageName(const QUrl& search)
 {
-    return search.scheme() == QLatin1String("snap") ? populate(m_socket.snapByName(search.host()), AbstractResource::Installed) : voidStream();
+    return search.scheme() == QLatin1String("snap") ? populate(m_client.find(QSnapdClient::MatchName, search.host())) : voidStream();
 }
 
-ResultsStream* SnapBackend::populate(SnapJob* job, AbstractResource::State state)
+template <class T>
+ResultsStream* SnapBackend::populate(T* job)
 {
     auto stream = new ResultsStream(QStringLiteral("Snap-populate"));
 
-    connect(job, &SnapJob::finished, stream, [stream, this, state](SnapJob* job) {
-        if (!job->isSuccessful()) {
-            stream->deleteLater();
+    connect(job, &QSnapdFindRequest::complete, stream, [stream, this, job]() {
+        if (job->error()) {
+            qDebug() << "error:" << job->error() << job->errorString();
+            stream->finish();
             return;
         }
 
-        const auto snaps = job->result().toArray();
-
         QVector<AbstractResource*> ret;
         QSet<SnapResource*> resources;
-        for(const auto& snap: snaps) {
-            const auto snapObj = snap.toObject();
-            const auto snapid = snapObj.value(QLatin1String("name")).toString();
-            SnapResource* res = m_resources.value(snapid);
+        for (int i=0, c=job->snapCount(); i<c; ++i) {
+            QSharedPointer<QSnapdSnap> snap(job->snap(i));
+            const auto snapname = snap->name();
+            SnapResource* res = m_resources.value(snapname);
             if (!res) {
-                res = new SnapResource(snapObj, state, this);
-                Q_ASSERT(res->packageName() == snapid);
+                res = new SnapResource(snap, AbstractResource::None, this);
+                Q_ASSERT(res->packageName() == snapname);
                 resources += res;
+            } else {
+                res->setSnap(snap);
             }
             ret += res;
         }
 
-        if (!resources.isEmpty()) {
-            foreach(SnapResource* res, resources)
-                m_resources[res->packageName()] = res;
-        }
+        foreach(SnapResource* res, resources)
+            m_resources[res->packageName()] = res;
+
         if (!ret.isEmpty())
             stream->resourcesFound(ret);
-        stream->deleteLater();
+        stream->finish();
     });
-    job->start();
+    job->runAsync();
     return stream;
 }
 
@@ -130,26 +151,40 @@ AbstractReviewsBackend* SnapBackend::reviewsBackend() const
     return m_reviews;
 }
 
-void SnapBackend::installApplication(AbstractResource* app, const AddonList& addons)
+Transaction* SnapBackend::installApplication(AbstractResource* app, const AddonList& addons)
 {
     Q_ASSERT(addons.isEmpty());
-    installApplication(app);
+    return installApplication(app);
 }
 
-void SnapBackend::installApplication(AbstractResource* _app)
+Transaction* SnapBackend::installApplication(AbstractResource* _app)
 {
-	TransactionModel *transModel = TransactionModel::global();
     auto app = qobject_cast<SnapResource*>(_app);
-    auto job = m_socket.snapAction(app->packageName(), SnapSocket::Install);
-	transModel->addTransaction(new SnapTransaction(app, job, &m_socket, Transaction::InstallRole));
+	return new SnapTransaction(app, m_client.install(app->packageName()), Transaction::InstallRole, AbstractResource::Installed);
 }
 
-void SnapBackend::removeApplication(AbstractResource* _app)
+Transaction* SnapBackend::removeApplication(AbstractResource* _app)
 {
-	TransactionModel *transModel = TransactionModel::global();
     auto app = qobject_cast<SnapResource*>(_app);
-    auto job = m_socket.snapAction(app->packageName(), SnapSocket::Remove);
-	transModel->addTransaction(new SnapTransaction(app, job, &m_socket, Transaction::RemoveRole));
+	return new SnapTransaction(app, m_client.remove(app->packageName()), Transaction::RemoveRole, AbstractResource::None);
+}
+
+QString SnapBackend::displayName() const
+{
+    return QStringLiteral("Snap");
+}
+
+void SnapBackend::refreshStates()
+{
+    auto ret = new StoredResultsStream({populate(m_client.list())});
+    connect(ret, &StoredResultsStream::finished, this, [this, ret](){
+        for (auto res: m_resources) {
+            if (ret->resources().contains(res))
+                res->setState(AbstractResource::Installed);
+            else
+                res->setState(AbstractResource::None);
+        }
+    });
 }
 
 #include "SnapBackend.moc"
