@@ -27,15 +27,17 @@
 #include <ReviewsBackend/AbstractReviewsBackend.h>
 #include <Transaction/Transaction.h>
 #include <DiscoverBackendsFactory.h>
-#include <KActionCollection>
 #include "Transaction/TransactionModel.h"
 #include "Category/CategoryModel.h"
+#include "utils.h"
 #include <QDebug>
 #include <QCoreApplication>
 #include <QThread>
 #include <QAction>
 #include <QMetaProperty>
 #include <KLocalizedString>
+#include <KSharedConfig>
+#include <KConfigGroup>
 
 ResourcesModel *ResourcesModel::s_self = nullptr;
 
@@ -48,11 +50,13 @@ ResourcesModel *ResourcesModel::global()
 
 ResourcesModel::ResourcesModel(QObject* parent, bool load)
     : QObject(parent)
+    , m_isFetching(false)
     , m_initializingBackends(0)
-    , m_actionCollection(nullptr)
+    , m_currentApplicationBackend(nullptr)
 {
     init(load);
-    connect(this, &ResourcesModel::allInitialized, this, &ResourcesModel::fetchingChanged);
+    connect(this, &ResourcesModel::allInitialized, this, &ResourcesModel::slotFetching);
+    connect(this, &ResourcesModel::backendsChanged, this, &ResourcesModel::initApplicationsBackend);
 }
 
 void ResourcesModel::init(bool load)
@@ -64,16 +68,11 @@ void ResourcesModel::init(bool load)
         QMetaObject::invokeMethod(this, "registerAllBackends", Qt::QueuedConnection);
 
 
-    QAction* updateAction = new QAction(this);
-    updateAction->setIcon(QIcon::fromTheme(QStringLiteral("system-software-update")));
-    updateAction->setText(i18nc("@action Checks the Internet for updates", "Check for Updates"));
-    updateAction->setShortcut(QKeySequence(Qt::CTRL + Qt::Key_R));
-    connect(this, &ResourcesModel::fetchingChanged, updateAction, [updateAction, this](){
-        updateAction->setEnabled(!isFetching());
-    });
-    connect(updateAction, &QAction::triggered, this, &ResourcesModel::checkForUpdates);
-
-    m_ownActions += updateAction;
+    m_updateAction = new QAction(this);
+    m_updateAction->setIcon(QIcon::fromTheme(QStringLiteral("system-software-update")));
+    m_updateAction->setText(i18nc("@action Checks the Internet for updates", "Check for Updates"));
+    m_updateAction->setShortcut(QKeySequence(Qt::CTRL + Qt::Key_R));
+    connect(m_updateAction, &QAction::triggered, this, &ResourcesModel::checkForUpdates);
 }
 
 ResourcesModel::ResourcesModel(const QString& backendName, QObject* parent)
@@ -105,8 +104,6 @@ void ResourcesModel::addResourcesBackend(AbstractResourcesBackend* backend)
     } else {
         m_initializingBackends++;
     }
-    if(m_actionCollection)
-        backend->integrateActions(m_actionCollection);
 
     connect(backend, &AbstractResourcesBackend::fetchingChanged, this, &ResourcesModel::callerFetchingChanged);
     connect(backend, &AbstractResourcesBackend::allDataChanged, this, &ResourcesModel::updateCaller);
@@ -114,12 +111,12 @@ void ResourcesModel::addResourcesBackend(AbstractResourcesBackend* backend)
     connect(backend, &AbstractResourcesBackend::updatesCountChanged, this, &ResourcesModel::updatesCountChanged);
     connect(backend, &AbstractResourcesBackend::resourceRemoved, this, &ResourcesModel::resourceRemoved);
     connect(backend, &AbstractResourcesBackend::passiveMessage, this, &ResourcesModel::passiveMessage);
-    connect(backend->backendUpdater(), &AbstractBackendUpdater::progressingChanged, this, &ResourcesModel::fetchingChanged);
+    connect(backend->backendUpdater(), &AbstractBackendUpdater::progressingChanged, this, &ResourcesModel::slotFetching);
 
     if(m_initializingBackends==0)
         emit allInitialized();
     else
-        emit fetchingChanged();
+        slotFetching();
 }
 
 void ResourcesModel::callerFetchingChanged()
@@ -131,6 +128,7 @@ void ResourcesModel::callerFetchingChanged()
         int idx = m_backends.indexOf(backend);
         Q_ASSERT(idx>=0);
         m_backends.removeAt(idx);
+//         Q_EMIT backendsChanged();
         CategoryModel::global()->blacklistPlugin(backend->name());
         backend->deleteLater();
         return;
@@ -138,13 +136,13 @@ void ResourcesModel::callerFetchingChanged()
 
     if(backend->isFetching()) {
         m_initializingBackends++;
-        emit fetchingChanged();
+        slotFetching();
     } else {
         m_initializingBackends--;
         if(m_initializingBackends==0)
             emit allInitialized();
         else
-            emit fetchingChanged();
+            slotFetching();
     }
 }
 
@@ -171,19 +169,30 @@ int ResourcesModel::updatesCount() const
     return ret;
 }
 
+bool ResourcesModel::hasSecurityUpdates() const
+{
+    bool ret = false;
+
+    foreach(AbstractResourcesBackend* backend, m_backends) {
+        ret |= backend->hasSecurityUpdates();
+    }
+
+    return ret;
+}
+
 void ResourcesModel::installApplication(AbstractResource* app)
 {
-    app->backend()->installApplication(app);
+    TransactionModel::global()->addTransaction(app->backend()->installApplication(app));
 }
 
 void ResourcesModel::installApplication(AbstractResource* app, const AddonList& addons)
 {
-    app->backend()->installApplication(app, addons);
+    TransactionModel::global()->addTransaction(app->backend()->installApplication(app, addons));
 }
 
 void ResourcesModel::removeApplication(AbstractResource* app)
 {
-    app->backend()->removeApplication(app);
+    TransactionModel::global()->addTransaction(app->backend()->removeApplication(app));
 }
 
 void ResourcesModel::registerAllBackends()
@@ -210,38 +219,28 @@ void ResourcesModel::registerBackendByName(const QString& name)
     emit backendsChanged();
 }
 
-void ResourcesModel::integrateActions(KActionCollection* w)
-{
-    Q_ASSERT(w->thread()==thread());
-    m_actionCollection = w;
-    setParent(w);
-    foreach(AbstractResourcesBackend* b, m_backends) {
-        b->integrateActions(w);
-    }
-}
-
 bool ResourcesModel::isFetching() const
 {
+    return m_isFetching;
+}
+
+void ResourcesModel::slotFetching()
+{
+    bool newFetching = false;
     foreach(AbstractResourcesBackend* b, m_backends) {
         // isFetching should sort of be enough. However, sometimes the backend itself
         // will still be operating on things, which from a model point of view would
         // still mean something going on. So, interpret that as fetching as well, for
         // the purposes of this property.
         if(b->isFetching() || (b->backendUpdater() && b->backendUpdater()->isProgressing())) {
-            return true;
+            newFetching = true;
+            break;
         }
     }
-    return false;
-}
-
-QList<QAction*> ResourcesModel::messageActions() const
-{
-    QList<QAction*> ret = m_ownActions;
-    foreach(AbstractResourcesBackend* b, m_backends) {
-        ret += b->messageActions();
+    if (newFetching != m_isFetching) {
+        m_isFetching = newFetching;
+        Q_EMIT fetchingChanged(m_isFetching);
     }
-    Q_ASSERT(!ret.contains(nullptr));
-    return ret;
 }
 
 bool ResourcesModel::isBusy() const
@@ -266,7 +265,7 @@ AggregatedResultsStream::AggregatedResultsStream(const QSet<ResultsStream*>& str
     Q_ASSERT(!streams.contains(nullptr));
     if (streams.isEmpty()) {
         qWarning() << "no streams to aggregate!!";
-        destruction(nullptr);
+        QTimer::singleShot(0, this, &AggregatedResultsStream::clear);
     }
 
     for (auto stream: streams) {
@@ -281,9 +280,9 @@ AggregatedResultsStream::AggregatedResultsStream(const QSet<ResultsStream*>& str
 
 void AggregatedResultsStream::addResults(const QVector<AbstractResource *>& res)
 {
-     m_results += res;
+    m_results += res;
 
-     m_delayedEmission.start();
+    m_delayedEmission.start();
 }
 
 void AggregatedResultsStream::emitResults()
@@ -299,6 +298,11 @@ void AggregatedResultsStream::emitResults()
 void AggregatedResultsStream::destruction(QObject* obj)
 {
     m_streams.remove(obj);
+    clear();
+}
+
+void AggregatedResultsStream::clear()
+{
     if (m_streams.isEmpty()) {
         emitResults();
         Q_EMIT finished();
@@ -308,19 +312,17 @@ void AggregatedResultsStream::destruction(QObject* obj)
 
 AggregatedResultsStream * ResourcesModel::findResourceByPackageName(const QUrl& search)
 {
-    QSet<ResultsStream*> streams;
-    foreach(auto backend, m_backends) {
-        streams << backend->findResourceByPackageName(search);
-    }
+    auto streams = kTransform<QSet<ResultsStream*>>(m_backends, [search](AbstractResourcesBackend* backend){ return backend->findResourceByPackageName(search); });
     return new AggregatedResultsStream(streams);
 }
 
 AggregatedResultsStream* ResourcesModel::search(const AbstractResourcesBackend::Filters& search)
 {
-    QSet<ResultsStream*> streams;
-    foreach(auto backend, m_backends) {
-        streams << backend->search(search);
+    if (search.isEmpty()) {
+        return new AggregatedResultsStream ({new ResultsStream(QStringLiteral("emptysearch"), {})});
     }
+
+    auto streams = kTransform<QSet<ResultsStream*>>(m_backends, [search](AbstractResourcesBackend* backend){ return backend->search(search); });
     return new AggregatedResultsStream(streams);
 }
 
@@ -338,5 +340,52 @@ AbstractResource* ResourcesModel::resourceForFile(const QUrl& file)
 void ResourcesModel::checkForUpdates()
 {
     for(auto backend: qAsConst(m_backends))
-        backend->checkForUpdates();
+        if (!backend->isFetching())
+            backend->checkForUpdates();
+}
+
+QVector<AbstractResourcesBackend *> ResourcesModel::applicationBackends() const
+{
+    return kFilter<QVector<AbstractResourcesBackend*>>(m_backends, [](AbstractResourcesBackend* b){ return b->hasApplications(); });
+}
+
+QVariantList ResourcesModel::applicationBackendsVariant() const
+{
+    return kTransform<QVariantList>(applicationBackends(), [](AbstractResourcesBackend* b) {return QVariant::fromValue<QObject*>(b);});
+}
+
+AbstractResourcesBackend* ResourcesModel::currentApplicationBackend() const
+{
+    return m_currentApplicationBackend;
+}
+
+void ResourcesModel::setCurrentApplicationBackend(AbstractResourcesBackend* backend, bool write)
+{
+    if (backend != m_currentApplicationBackend) {
+        if (write) {
+            KConfigGroup settings(KSharedConfig::openConfig(), "ResourcesModel");
+            if (backend)
+                settings.writeEntry("currentApplicationBackend", backend->name());
+            else
+                settings.deleteEntry("currentApplicationBackend");
+        }
+
+        qDebug() << "setting currentApplicationBackend" << backend;
+        m_currentApplicationBackend = backend;
+        Q_EMIT currentApplicationBackendChanged(backend);
+    }
+}
+
+void ResourcesModel::initApplicationsBackend()
+{
+    KConfigGroup settings(KSharedConfig::openConfig(), "ResourcesModel");
+    const QString name = settings.readEntry<QString>("currentApplicationBackend", QStringLiteral("packagekit-backend"));
+
+    const auto backends = applicationBackends();
+    auto idx = kIndexOf(backends, [name](AbstractResourcesBackend* b) { return b->name() == name; });
+    if (idx<0) {
+        idx = kIndexOf(backends, [](AbstractResourcesBackend* b) { return b->hasApplications(); });
+        qDebug() << "falling back applications backend to" << idx;
+    }
+    setCurrentApplicationBackend(backends.value(idx, nullptr), false);
 }
