@@ -25,6 +25,7 @@
 #include "AbstractResourcesBackend.h"
 #include <QSet>
 #include <QDateTime>
+#include <QTimer>
 
 class AbstractResourcesBackend;
 
@@ -38,7 +39,6 @@ class DISCOVERCOMMON_EXPORT StandardBackendUpdater : public AbstractBackendUpdat
         bool hasUpdates() const override;
         qreal progress() const override;
         void start() override;
-        long unsigned int remainingTime() const override;
         
         QList<AbstractResource*> toUpdate() const override;
         void addResources(const QList<AbstractResource*>& apps) override;
@@ -47,11 +47,8 @@ class DISCOVERCOMMON_EXPORT StandardBackendUpdater : public AbstractBackendUpdat
         QDateTime lastUpdate() const override;
         bool isCancelable() const override;
         bool isProgressing() const override;
-        QString statusDetail() const override;
-        QString statusMessage() const override;
-        quint64 downloadSpeed() const override;
         bool isMarked(AbstractResource* res) const override;
-        void setStatusDetail(const QString& message);
+        double updateSize() const override;
         void setProgress(qreal p);
         int updatesCount() const;
 
@@ -73,9 +70,9 @@ class DISCOVERCOMMON_EXPORT StandardBackendUpdater : public AbstractBackendUpdat
         AbstractResourcesBackend * const m_backend;
         QSet<AbstractResource*> m_pendingResources;
         bool m_settingUp;
-        QString m_statusDetail;
         qreal m_progress;
         QDateTime m_lastUpdate;
+        QTimer m_timer;
 };
 
 #endif // STANDARDBACKENDUPDATER_H
